@@ -89,7 +89,178 @@ static gboolean src_process_space 	(gchar *space);
 static gboolean src_process_modifier 	(gchar *modifier);
 static gboolean src_process_cursor 	(gchar *cursor);
 
+static void
+src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
+{
+    gchar *txt;
 
+    srl_assert (src_use_braille || src_use_braille_monitor);
+
+	
+    src_enable_format_string = FALSE; 
+    txt = src_presentation_sro_get_pres_for_device (obj, reason, "braille");
+    if (txt)
+    {
+	gchar *tmp;
+	gchar *cursor = NULL;
+	SRObjectRoles role;
+	gchar *status = NULL;
+	gchar offset[5];
+
+	sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+	if (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL)
+	{
+	    SRLong offset;
+	    SRTextAttribute *attr;
+	    gchar status_[5] = "    ";
+
+	    if (sro_text_get_caret_offset (obj, &offset, SR_INDEX_CONTAINER))
+		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset, src_braille_cursor_style);
+	    
+	    if (sro_text_get_text_attr_from_caret (obj, SR_TEXT_BOUNDARY_CHAR, 
+				    &attr, SR_INDEX_CONTAINER))
+	    {
+		gchar *tmp;
+		if (sra_get_attribute_value (attr[0], "bold", &tmp))
+		{
+		    if (strcmp (tmp, "true") == 0)
+			status_[0] = 'B';
+		    SR_freeString (tmp);
+		}
+	    	if (sra_get_attribute_value (attr[0], "italic", &tmp))
+		{
+		    if (strcmp (tmp, "true") == 0)
+		        status_[1] = 'I';
+		    SR_freeString (tmp);
+		}
+	    	if (sra_get_attribute_value (attr[0], "underline", &tmp))
+		{
+		    if (strcmp (tmp, "single") == 0 || strcmp (tmp, "double") == 0)
+		        status_[2] = 'U';
+		    SR_freeString (tmp);
+		}
+	    	if (sra_get_attribute_value (attr[0], "selected", &tmp))
+		{
+		    if (strcmp (tmp, "true") == 0)
+		        status_[3] = 'S';
+		    SR_freeString (tmp);
+		}
+		sra_free (attr);
+		status = g_strconcat("<BRLDISP role=\"status\" start=\"0\" clear=\"true\">"
+					 "<TEXT>",
+					 status_,
+					 "</TEXT>"
+					 "</BRLDISP>", 
+					 NULL);
+	    }
+	}
+	sprintf (offset, "%d", src_braille_offset);
+	tmp = g_strconcat ("<BRLOUT language=\"",src_braille_translation_table,"\" bstyle=\"", src_braille_style,"\" clear=\"true\">",
+			"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\"",
+			cursor ? cursor : "",
+			">",
+			txt,
+			"</BRLDISP>",
+			status ? status : "",
+			"</BRLOUT>",
+			NULL);
+	/*fprintf (stderr, "\n%s", tmp);*/
+	if (src_use_braille)
+	    src_braille_send (tmp);
+	if (src_use_braille_monitor)
+	    src_brlmon_send (tmp);
+	g_free (tmp);
+	g_free (cursor);
+    }
+    g_free (txt);
+}
+
+
+static void
+src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
+{
+    SRObjectRoles role;
+
+    srl_assert (src_use_speech);
+
+    src_enable_format_string = TRUE; 
+    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+    if (strcmp (reason, "object:text-changed:insert") == 0
+	    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL))
+    {
+	gchar *diff;
+	if (sro_text_get_difference (obj, &diff, SR_INDEX_CONTAINER))
+	{
+	    if (diff[1] != '\0')
+	    {
+		if (src_last_key_echo)
+		{
+		    g_free (src_last_key_echo);
+		    src_last_key_echo = NULL;
+		}
+		src_last_key_echo = diff;
+		src_cmd_queue_add ("kbd key");
+		src_cmd_queue_process ();
+		src_last_key_echo = NULL;
+	    }
+	    else
+	    {
+		if (g_ascii_ispunct (diff[0]))
+    		    src_process_punct (diff);
+		else if (g_ascii_isspace (diff[0]))
+    		    src_process_space (diff);
+		else
+    		    src_process_key_ (diff);
+	    }
+	    SR_freeString (diff);
+	}
+    }
+    else
+    {
+	gchar *txt;
+    	if (src_last_key_echo)
+	{
+	    g_free (src_last_key_echo);
+	    src_last_key_echo = NULL;
+	}
+	txt = src_presentation_sro_get_pres_for_device (obj, reason, "speech");
+	if (txt && txt[0])
+	{
+	    SRObjectRoles role;
+	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+	    if ( role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
+	    else if ((role == SR_ROLE_TABLE || role == SR_ROLE_TREE_TABLE) &&
+		strcmp (reason , "focus:") == 0)
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
+	    else
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
+	}
+	g_free (txt);
+    }
+}
+
+static void
+src_present_real_sro_for_magnifier (SRObject *obj, gchar *reason)
+{
+    gchar *txt;
+
+    srl_assert (src_use_magnifier);
+
+    txt = src_presentation_sro_get_pres_for_device (obj, reason, "magnifier");
+    if (txt)
+    {
+        gchar *tmp;
+        tmp = g_strconcat ("<MAGOUT><ZOOMER ID=\"generic_zoomer\" tracking=\"focus\" ",
+			txt,
+			"></ZOOMER></MAGOUT>",
+			NULL);
+        if (tmp)
+	    src_magnifier_send (tmp);
+	g_free (tmp);
+	g_free (txt);
+    }
+}
 
 void
 src_present_real_sro (SRObject *obj)
@@ -108,175 +279,16 @@ src_present_real_sro (SRObject *obj)
     reason = NULL;
     if (!sro_get_reason (obj, &reason))
 	return;
-    
+
     if (src_use_braille || src_use_braille_monitor)
-    {
-	gchar *txt;
-	
-	src_enable_format_string = FALSE; 
-    	txt = src_presentation_sro_get_pres_for_device (obj, reason, "braille");
-	if (txt)
-	{
-	    gchar *tmp;
-	    gchar *cursor = NULL;
-	    SRObjectRoles role;
-	    gchar *status = NULL;
-	    gchar offset[5];
-
-	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    if (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL)
-	    {
-		SRLong offset;
-		SRTextAttribute *attr;
-		gchar status_[5] = "    ";
-
-		if (sro_text_get_caret_offset (obj, &offset, SR_INDEX_CONTAINER))
-		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset, src_braille_cursor_style);
-	    
-		if (sro_text_get_text_attr_from_caret (obj, SR_TEXT_BOUNDARY_CHAR, 
-				    &attr, SR_INDEX_CONTAINER))
-		{
-		    gchar *tmp;
-		    if (sra_get_attribute_value (attr[0], "bold", &tmp))
-		    {
-			if (strcmp (tmp, "true") == 0)
-			    status_[0] = 'B';
-			SR_freeString (tmp);
-		    }
-	    	    if (sra_get_attribute_value (attr[0], "italic", &tmp))
-		    {
-			if (strcmp (tmp, "true") == 0)
-			    status_[1] = 'I';
-			SR_freeString (tmp);
-		    }
-	    	    if (sra_get_attribute_value (attr[0], "underline", &tmp))
-		    {
-			if (strcmp (tmp, "single") == 0 || strcmp (tmp, "double") == 0)
-			    status_[2] = 'U';
-			SR_freeString (tmp);
-		    }
-	    	    if (sra_get_attribute_value (attr[0], "selected", &tmp))
-		    {
-			if (strcmp (tmp, "true") == 0)
-			    status_[3] = 'S';
-			SR_freeString (tmp);
-		    }
-		    sra_free (attr);
-		    status = g_strconcat("<BRLDISP role=\"status\" start=\"0\" clear=\"true\">"
-					 "<TEXT>",
-					 status_,
-					 "</TEXT>"
-					 "</BRLDISP>", 
-					 NULL);
-		}
-	    }
-	    sprintf (offset, "%d", src_braille_offset);
-	    tmp = g_strconcat ("<BRLOUT language=\"",src_braille_translation_table,"\" bstyle=\"", src_braille_style,"\" clear=\"true\">",
-				"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\"",
-				cursor ? cursor : "",
-				">",
-				txt,
-				"</BRLDISP>",
-				status ? status : "",
-				"</BRLOUT>",
-				NULL);
-	    /*fprintf (stderr, "\n%s", tmp);*/
-	    if (src_use_braille)
-		src_braille_send (tmp);
-	    if (src_use_braille_monitor)
-		src_brlmon_send (tmp);
-	    g_free (tmp);
-	    g_free (cursor);
-	}
-	g_free (txt);
-    }
+	src_present_real_sro_for_brl_and_brlmon (obj, reason);
     
     if (src_use_speech)
-    {
-	src_enable_format_string = TRUE; 
-	{
-	    SRObjectRoles role;
-	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    if (strcmp (reason, "object:text-changed:insert") == 0
-		    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL))
-	    {
-		gchar *diff;
-		if (sro_text_get_difference (obj, &diff, SR_INDEX_CONTAINER))
-		{
-		    if (diff[1] != '\0')
-		    {
-			if (src_last_key_echo)
-			{
-			    g_free (src_last_key_echo);
-			    src_last_key_echo = NULL;
-			}
-			src_last_key_echo = diff;
-			src_cmd_queue_add ("kbd key");
-			src_cmd_queue_process ();
-			src_last_key_echo = NULL;
-		    }
-		    else
-		    {
-			if (g_ascii_ispunct (diff[0]))
-	    		    src_process_punct (diff);
-			else if (g_ascii_isspace (diff[0]))
-	    		    src_process_space (diff);
-			else
-	    		    src_process_key_ (diff);
-		    }
-		    SR_freeString (diff);
-		}
-	    }
-	    else
-	    {
-		gchar *txt;
-
-	    	if (src_last_key_echo)
-		{
-		    g_free (src_last_key_echo);
-		    src_last_key_echo = NULL;
-		}
-		txt = src_presentation_sro_get_pres_for_device (obj, reason, "speech");
-    		if (txt && txt[0])
-    		{
-		    SRObjectRoles role;
-		    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-		    if ( role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
-			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
-		    else if ((role == SR_ROLE_TABLE || role == SR_ROLE_TREE_TABLE) &&
-				strcmp (reason , "focus:") == 0)
-			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
-		    else
-			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
-		}
-		g_free (txt);
-	    }
-	}
-    }
-        
+	src_present_real_sro_for_speech (obj, reason);
 
     if (src_use_magnifier)
-    {
-	gchar *txt;
-	txt = src_presentation_sro_get_pres_for_device (obj, reason, "magnifier");
-	if (txt)
-	{
-/*	    fprintf (stderr, "\n(%s %d) txt = %s",
-			__FILE__,
-			__LINE__,
-			txt);
-*/
-	    gchar *tmp;
-	    tmp = g_strconcat ("<MAGOUT><ZOOMER ID=\"generic_zoomer\" tracking=\"focus\" ",
-				txt,
-				"></ZOOMER></MAGOUT>",
-				NULL);
-	    if (tmp)
-		src_magnifier_send (tmp);
-	    g_free (tmp);
-	    g_free (txt);
-	}
-    }
+	src_present_real_sro_for_magnifier (obj, reason);
+
     SR_freeString (reason);
 }
 
@@ -2248,18 +2260,20 @@ src_process_sro_event (SREvent *event,
 
     if (sre_get_event_data (event, (void**)&obj))
     {
+        if (src_use_magnifier)
+	{
+	    gchar *reason;
+	    if (sro_get_reason (obj, &reason))
+	    {
+		if (strcmp (reason, "object:visible-data-changed") == 0)
+		    src_present_real_sro_for_magnifier (obj, reason);
+		SR_freeString (reason);
+	    }	    
+	}
 	src_process_sro (obj);
 	if (src_use_magnifier)
 	    src_magnifier_start_panning (obj);
     }
-/*
-    {
-	gchar *r;
-	sro_get_reason (obj, &r);
-	fprintf (stderr, "\n%s", r);
-	SR_freeString (r);
-    }
-*/
 }
 
 static void
